(function ($) {
// START jQuery

Drupal.behaviors.sheetnode = function(context) {
  // Abort early if sheetnode settings not in place (which happens in case of Views style settings).
  if (typeof(Drupal.settings.sheetnode) == 'undefined') return;

  $(".sheetview", context).not(".sheetview-processed").each(function() {
    if (typeof(Drupal.settings.sheetnode[$(this).attr('id')]) != 'undefined') {
      Drupal.sheetnode.sheetviews.push(new Drupal.sheetnode(Drupal.settings.sheetnode[$(this).attr('id')], this, context));
    }
  });
}

Drupal.sheetnode = function(settings, container, context) {
  this.settings = settings;
  this.$container = $(container);
  this.context = context;
  var self = this;
  window.setTimeout(function(){ self.start(); }, 1);
}

Drupal.sheetnode.sheetviews = [];

Drupal.sheetnode.functionsSetup = function() {
  // ORG.DRUPAL.FIELD server-side function.
  SocialCalc.Formula.FunctionList["ORG.DRUPAL.FIELD"] = [Drupal.sheetnode.functionDrupalField, -1, "drupalfield", "", "drupal"];
  SocialCalc.Constants["s_fdef_ORG.DRUPAL.FIELD"] = 'Returns a field from the specified Drupal entity (node, user, etc.)';
  SocialCalc.Constants.s_farg_drupalfield = 'field-name, [oid, entity-name]';

  // ORG.DRUPAL.TOKEN server-side function.
  SocialCalc.Formula.FunctionList["ORG.DRUPAL.TOKEN"] = [Drupal.sheetnode.functionDrupalToken, -1, "drupaltoken", "", "drupal"];
  SocialCalc.Constants["s_fdef_ORG.DRUPAL.TOKEN"] = 'Returns a token value from the specified Drupal entity (node, user, etc.)';
  SocialCalc.Constants.s_farg_drupaltoken = 'token, [oid, entity-name]';

  // Update function classes.
  SocialCalc.Constants.function_classlist.push('drupal');
  SocialCalc.Constants.s_fclass_drupal = "Drupal";
}

Drupal.sheetnode.functionDrupalField = function(fname, operand, foperand, sheet) {
  var scf = SocialCalc.Formula;
  var oid, entity, field;

  field = scf.OperandValueAndType(sheet, foperand);
  oid = scf.OperandValueAndType(sheet, foperand);
  entity = scf.OperandValueAndType(sheet, foperand);
  if (isNaN(parseInt(oid.value))) {
    oid.value = Drupal.settings.sheetnode.context['oid'];
    entity.value = Drupal.settings.sheetnode.context['entity-name'];
  }

  $.ajax({
    type: 'POST',
    url: Drupal.settings.basePath+'sheetnode/field',
    data: 'oid='+oid.value+'&entity='+escape(entity.value)+'&field='+escape(field.value),
    datatype: 'json',
    async: false,
    success: function (data) {
      var result = Drupal.parseJson(data);
      operand.push(result);
    }
  });
}

Drupal.sheetnode.functionDrupalToken = function(fname, operand, foperand, sheet) {
  var scf = SocialCalc.Formula;
  var oid, entity, field;

  token = scf.OperandValueAndType(sheet, foperand);
  oid = scf.OperandValueAndType(sheet, foperand);
  entity = scf.OperandValueAndType(sheet, foperand);
  if (isNaN(parseInt(oid.value))) {
    oid.value = Drupal.settings.sheetnode.context['oid'];
    entity.value = Drupal.settings.sheetnode.context['entity-name'];
  }

  $.ajax({
    type: 'POST',
    url: Drupal.settings.basePath+'sheetnode/token',
    data: 'oid='+oid.value+'&entity='+escape(entity.value)+'&token='+escape(token.value),
    datatype: 'json',
    async: false,
    success: function (data) {
      var result = Drupal.parseJson(data);
      operand.push(result);
    }
  });
}

Drupal.sheetnode.loadsheetSetup = function() {
  SocialCalc.RecalcInfo.LoadSheet = function(sheetname) {
    data = $.ajax({
      type: 'POST',
      url: Drupal.settings.basePath+'sheetnode/load',
      data: 'sheetname='+escape(sheetname),
      datatype: 'text',
      async: false
    }).responseText;
    if (data !== null) {
      SocialCalc.RecalcLoadedSheet(sheetname, data, true);
    }
    return data;
  }
}

Drupal.sheetnode.prototype.focusSetup = function() {
  $('.form-text,.form-textarea,.form-select', this.context).not('.socialcalc-input').focus(function(e) {
    SocialCalc.CmdGotFocus(this);
  });
}

Drupal.sheetnode.viewModes = {
  readOnly: 0,
  fiddleMode: 1,
  htmlTable: 2
}

Drupal.sheetnode.prototype.start = function() {
  var self = this;
  var showEditor = this.settings.saveElement || this.settings.viewMode == Drupal.sheetnode.viewModes.fiddleMode;
  var showToolbar = this.settings.saveElement || (this.settings.showToolbar && this.settings.viewMode == Drupal.sheetnode.viewModes.fiddleMode);
  
  // SocialCalc initialization.
  SocialCalc.Popup.Controls = {};
  SocialCalc.ConstantsSetImagePrefix(this.settings.imagePrefix);
  SocialCalc.Constants.defaultCommentClass = "cellcomment";
  SocialCalc.Constants.defaultReadonlyClass = "readonly";
  this.spreadsheet = showEditor ? new SocialCalc.SpreadsheetControl() : new SocialCalc.SpreadsheetViewer();
  if (showToolbar) {
    // Remove unwanted tabs.
    this.spreadsheet.tabs.splice(this.spreadsheet.tabnums.clipboard, 1);
    this.spreadsheet.tabs.splice(this.spreadsheet.tabnums.audit, 1);
    if (!this.settings.permissions['edit sheetnode settings']) {
      this.spreadsheet.tabs.splice(this.spreadsheet.tabnums.settings, 1);
    }
    this.spreadsheet.tabnums = {};
    for (var i=0; i<this.spreadsheet.tabs.length; i++) {
      this.spreadsheet.tabnums[this.spreadsheet.tabs[i].name] = i;
    }
  }
  else {
    this.spreadsheet.tabbackground="display:none;";
    this.spreadsheet.toolbarbackground="display:none;";
  }

  // Trigger event to alert plugins that we've created the spreadsheet.
  this.$container.trigger('sheetnodeCreated', {spreadsheet: this.spreadsheet});

  // Read in data and recompute.
  var parts = this.spreadsheet.DecodeSpreadsheetSave(this.settings.value);
  if (parts && parts.sheet) {
    this.spreadsheet.ParseSheetSave(this.settings.value.substring(parts.sheet.start, parts.sheet.end));
  }
  if (showEditor) {
    this.spreadsheet.InitializeSpreadsheetControl(this.settings.containerElement, 700, this.$container.width());
  }
  else {
    this.spreadsheet.InitializeSpreadsheetViewer(this.settings.containerElement, 700, this.$container.width());
  }
  if (parts && parts.edit) {
    this.spreadsheet.editor.LoadEditorSettings(this.settings.value.substring(parts.edit.start, parts.edit.end));
  }
  if (!this.settings.saveElement && this.settings.viewMode == Drupal.sheetnode.viewModes.htmlTable) {
    this.$container.html(SocialCalc.SpreadsheetViewerCreateSheetHTML(this.spreadsheet));
  }

  // Special handling for Views AJAX.
  try {
    $('input[type=submit]', Drupal.settings.views.ajax.id).click(function() {
      self.save();
    });
  }
  catch (e) {
    // Do nothing.
  }

  // Call our setup functions.
  this.focusSetup();
  Drupal.sheetnode.functionsSetup();
  Drupal.sheetnode.loadsheetSetup();

  // DOM initialization.
  if (this.settings.saveElement) {
    this.$form = this.$container.parents('form').submit(function() {
      self.save();
      return true;
    });
  }
  $(window).resize(function() {
    self.resize();
  });
  $('div#SocialCalc-edittools', this.$container).parent('div').attr('id', 'SocialCalc-toolbar');
  $('td#SocialCalc-edittab', this.$container).parents('div:eq(0)').attr('id', 'SocialCalc-tabbar');
  $('input:text', this.$container).addClass('form-text socialcalc-input');
  $('input:radio', this.$container).addClass('form-radio socialcalc-input');
  $('input:checkbox', this.$container).addClass('form-checkbox socialcalc-input');
  $('textarea', this.$container).addClass('form-textarea socialcalc-input');
  $('select', this.$container).addClass('form-select socialcalc-input');
  $('input:button', this.$container).addClass('form-submit socialcalc-input');
  $('div#SocialCalc-sorttools td:first', this.$container).css('width', 'auto');
  $('div#SocialCalc-settingsview', this.$container).css('border', 'none').css('width', 'auto').css('height', 'auto');

  // Lock cells requires special permission.
  if (showToolbar && !this.settings.permissions['edit sheetnode settings']) {
    $('span#SocialCalc-locktools', this.$container).css('display', 'none');
  }

  // Prepare for fullscreen handling when clicking the SocialCalc icon.
  $('td#'+SocialCalc.Constants.defaultTableEditorIDPrefix+'logo img', this.$container).attr('title', Drupal.t('Fullscreen')).click(function() { self.fullscreen() });
  
  // Signal that we've processed this instance of sheetnode.
  this.$container.addClass('sheetview-processed');

  // Trigger event to alert plugins that we've built the spreadsheet.
  this.$container.trigger('sheetnodeReady', {spreadsheet: this.spreadsheet});

  // Force a recalc to refresh all values and scrollbars.
  this.spreadsheet.editor.EditorScheduleSheetCommands('recalc');
}

Drupal.sheetnode.prototype.fullscreen = function() {
  if (this.$container.hasClass('sheetview-fullscreen')) { // Going back to normal:
    // Restore saved values.
    this.$container.removeClass('sheetview-fullscreen');
    if (this.beforeFullscreen.index >= this.beforeFullscreen.parentElement.children().length) {
      this.beforeFullscreen.parentElement.append(this.$container);
    } else {
      this.$container.insertBefore(this.beforeFullscreen.parentElement.children().get(this.beforeFullscreen.index));
    }
    this.spreadsheet.requestedHeight = this.beforeFullscreen.requestedHeight;
    this.resize();
    $('body').css('overflow', 'auto');
    window.scroll(this.beforeFullscreen.x, this.beforeFullscreen.y);
  }
  else { // Going fullscreen:
    // Save current values.
    this.beforeFullscreen = {
      parentElement: this.$container.parent(),
      index: this.$container.parent().children().index(this.$container),
      x: $(window).scrollLeft(), y: $(window).scrollTop(),
      requestedHeight: this.spreadsheet.requestedHeight
    };

    // Set values needed to go fullscreen.
    $('body').append(this.$container).css('overflow', 'hidden');
    this.$container.addClass('sheetview-fullscreen');
    this.resize();
    window.scroll(0,0);
  }
}

Drupal.sheetnode.prototype.resize = function() {
  // Adjust width and height if needed.
  if (this.$container.hasClass('sheetview-fullscreen')) {
    this.spreadsheet.requestedHeight = this.$container.height();
  }
  this.spreadsheet.requestedWidth = this.$container.width();
  this.spreadsheet.DoOnResize();
}

Drupal.sheetnode.prototype.save = function() {
  var self = this;
  $('#'+this.settings.saveElement, this.$form).val(this.spreadsheet.CreateSpreadsheetSave());
  $('#edit-log', this.$form).each(function() {
    var audit = self.spreadsheet.sheet.CreateAuditString();
    var log = $(this).val();
    if (!log.length) {
      $(this).val(audit);
    }
    else {
      $(this).val(log + '\n' + audit);
    }
  });
}

// END jQuery
})(jQuery);

